@@ -2,7 +2,7 @@
                 routines, but not by user code
    Copyright (c) 2001-2015 Philip Kendall, Darren Salt
 
-   $Id: internals.h 5422 2016-04-29 12:54:59Z fredm $
+   $Id: internals.h 5728 2016-08-12 11:45:36Z fredm $
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -102,6 +102,11 @@
 #define strncasecmp _strnicmp
 #endif		/* #if !defined(HAVE_STRNCASECMP) && defined(HAVE__STRNICMP) */
 
+#ifndef MAX
+#define MAX(a,b)    (((a) > (b)) ? (a) : (b))
+#define MIN(a,b)    (((a) < (b)) ? (a) : (b))
+#endif
+
 /* Print using the user-provided error function */
 libspectrum_error
 libspectrum_print_error( libspectrum_error error, const char *format, ... )
@@ -133,6 +138,14 @@ libspectrum_error
 libspectrum_bzip2_inflate( const libspectrum_byte *bzptr, size_t bzlength,
 			   libspectrum_byte **outptr, size_t *outlength );
 
+libspectrum_error
+libspectrum_zip_inflate( const libspectrum_byte *zipptr, size_t ziplength,
+			  libspectrum_byte **outptr, size_t *outlength );
+
+libspectrum_error
+libspectrum_zip_blind_read( const libspectrum_byte *zipptr, size_t ziplength,
+                            libspectrum_byte **outptr, size_t *outlength );
+
 /* The TZX file signature */
 
 extern const char * const libspectrum_tzx_signature;
@@ -287,6 +300,9 @@ void
 libspectrum_set_pause_tstates( libspectrum_tape_block *block,
                                libspectrum_dword pause_tstates );
 
+size_t
+libspectrum_bits_to_bytes( size_t bits );
+
 extern const int LIBSPECTRUM_BITS_IN_BYTE;
 
 /* glib replacement functions */
