@@ -2,7 +2,7 @@
    Copyright (c) 2001, 2002 Philip Kendall, Darren Salt
    Copyright (c) 2011-2015 Fredrick Meunier
 
-   $Id: pzx_read.c 5422 2016-04-29 12:54:59Z fredm $
+   $Id: pzx_read.c 5775 2016-08-23 18:20:55Z pak21 $
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -26,7 +26,6 @@
 
 #include <config.h>
 
-#include <math.h>
 #include <stddef.h>
 #include <stdio.h>
 #include <string.h>
@@ -41,9 +40,6 @@ typedef struct pzx_context {
 
 } pzx_context;
 
-static const libspectrum_byte PZX_VERSION_MAJOR = 1;
-static const libspectrum_byte PZX_VERSION_MINOR = 0;
-
 /* Constants etc for each block type */
 
 #define PZX_HEADER "PZXT"
@@ -258,7 +254,7 @@ read_data_block( libspectrum_tape *tape, const libspectrum_byte **buffer,
   count = libspectrum_read_dword( buffer );
   initial_level = !!(count & 0x80000000);
   count &= 0x7fffffff;
-  count_bytes = ceil( count / (double)LIBSPECTRUM_BITS_IN_BYTE );
+  count_bytes = libspectrum_bits_to_bytes( count );
   bits_in_last_byte =
     count % LIBSPECTRUM_BITS_IN_BYTE ?
       count % LIBSPECTRUM_BITS_IN_BYTE : LIBSPECTRUM_BITS_IN_BYTE;
