@@ -1,7 +1,7 @@
 /* tap.c: Routines for handling .tap files
    Copyright (c) 2001-2003 Philip Kendall
 
-   $Id: tap.c 4440 2011-05-15 10:53:47Z fredm $
+   $Id: tap.c 5124 2015-03-02 19:51:36Z zubzero $
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -102,7 +102,7 @@ internal_tap_read( libspectrum_tape *tape, const libspectrum_byte *buffer,
     }
 
     /* Allocate memory for the data */
-    data = libspectrum_malloc( data_length * sizeof( *data ) );
+    data = libspectrum_new( libspectrum_byte, data_length );
     libspectrum_tape_block_set_data( block, data );
 
     /* Copy the block data across */
