@@ -1,7 +1,7 @@
 /* tape_block.c: one tape block
    Copyright (c) 2003-2016 Philip Kendall
 
-   $Id: tape_block.c 5422 2016-04-29 12:54:59Z fredm $
+   $Id: tape_block.c 5725 2016-08-11 12:18:13Z fredm $
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -244,7 +244,7 @@ libspectrum_tape_block_init( libspectrum_tape_block *block,
   case LIBSPECTRUM_TAPE_BLOCK_PULSE_SEQUENCE:
     state->block_state.pulse_sequence.index = 0;
     state->block_state.pulse_sequence.pulse_count = 0;
-    state->block_state.pulse_sequence.level = 1;
+    state->block_state.pulse_sequence.level = -1;
     return LIBSPECTRUM_ERROR_NONE;
   case LIBSPECTRUM_TAPE_BLOCK_DATA_BLOCK:
     return data_block_init( &(block->types.data_block),
@@ -348,9 +348,6 @@ static libspectrum_error
 generalised_data_init( libspectrum_tape_generalised_data_block *block,
                        libspectrum_tape_generalised_data_block_state *state )
 {
-  state->state = block->pilot_table.symbols_in_block ?
-    LIBSPECTRUM_TAPE_STATE_PILOT : LIBSPECTRUM_TAPE_STATE_DATA1;
-
   state->run = 0;
   state->symbols_through_run = 0;
   state->edges_through_symbol = 0;
@@ -362,6 +359,16 @@ generalised_data_init( libspectrum_tape_generalised_data_block *block,
   state->bits_through_byte = 0;
   state->bytes_through_stream = 0;
 
+  if( block->pilot_table.symbols_in_block ) {
+    state->state = LIBSPECTRUM_TAPE_STATE_PILOT;
+  } else if( block->data_table.symbols_in_block ) {
+    state->state = LIBSPECTRUM_TAPE_STATE_DATA1;
+    state->current_byte = block->data[ 0 ];
+    state->current_symbol = get_generalised_data_symbol( block, state );
+  } else {
+    state->state = LIBSPECTRUM_TAPE_STATE_PAUSE;
+  }
+
   return LIBSPECTRUM_ERROR_NONE;
 }
 
