@@ -2,7 +2,7 @@
    Copyright (c) 2002 Darren Salt, Philip Kendall
    Copyright (c) 2015 Stuart Brady
 
-   $Id: zlib.c 5422 2016-04-29 12:54:59Z fredm $
+   $Id: zlib.c 5718 2016-08-07 12:21:23Z fredm $
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -28,8 +28,6 @@
 
 #include <config.h>
 
-#ifdef HAVE_ZLIB_H
-
 #include <errno.h>
 #include <stdio.h>
 #include <string.h>
@@ -78,6 +76,13 @@ libspectrum_gzip_inflate( const libspectrum_byte *gzptr, size_t gzlength,
   return zlib_inflate( gzptr, gzlength, outptr, outlength, 1 );
 }
 
+libspectrum_error
+libspectrum_zip_inflate( const libspectrum_byte *zipptr, size_t ziplength,
+                         libspectrum_byte **outptr, size_t *outlength )
+{
+  return zlib_inflate( zipptr, ziplength, outptr, outlength, 1 );
+}
+
 static libspectrum_error
 zlib_inflate( const libspectrum_byte *gzptr, size_t gzlength,
 	      libspectrum_byte **outptr, size_t *outlength, int gzip_hack )
@@ -354,5 +359,3 @@ libspectrum_zlib_compress( const libspectrum_byte *data, size_t length,
     return LIBSPECTRUM_ERROR_LOGIC;
   }
 }
-
-#endif				/* #ifdef HAVE_ZLIB_H */
